@@ -7,7 +7,9 @@ use ::std::fmt::{Write as _};
 use ::std::fs;
 use ::std::io;
 use ::std::io::{Write as _};
+use ::std::iter::Iterator;
 use ::std::path::{Path, PathBuf};
+use ::std::collections::BTreeSet;
 
 use ::sass_rs as sass;
 use ::walkdir;
@@ -31,6 +33,7 @@ pub struct BuilderConfiguration {
 	pub js_route_base : Option<PathBuf>,
 	
 	pub images_route_base : Option<PathBuf>,
+	pub icons_route_base : Option<PathBuf>,
 	pub favicons_route_base : Option<PathBuf>,
 	pub fonts_route_base : Option<PathBuf>,
 	
@@ -63,6 +66,7 @@ impl Default for BuilderConfiguration {
 		let _js_route_base = Some (PathBuf::from ("/assets/js"));
 		
 		let _images_route_base = Some (PathBuf::from ("/assets/images"));
+		let _icons_route_base = Some (PathBuf::from ("/assets/icons"));
 		let _favicons_route_base = Some (PathBuf::from ("/assets/favicons"));
 		let _fonts_route_base = Some (PathBuf::from ("/assets/fonts"));
 		
@@ -85,6 +89,7 @@ impl Default for BuilderConfiguration {
 				js_route_base : _js_route_base,
 				
 				images_route_base : _images_route_base,
+				icons_route_base : _icons_route_base,
 				favicons_route_base : _favicons_route_base,
 				fonts_route_base : _fonts_route_base,
 				
@@ -117,6 +122,7 @@ impl BuilderConfiguration {
 				js_route_base : None,
 				
 				images_route_base : None,
+				icons_route_base : None,
 				favicons_route_base : None,
 				fonts_route_base : None,
 				
@@ -130,7 +136,7 @@ impl BuilderConfiguration {
 		if ! _sources.is_dir () {
 			panic! ("[148bc689]");
 		}
-		_sources
+		normalize_path (&_sources)
 	}
 	
 	pub fn resolve_outputs () -> PathBuf {
@@ -138,7 +144,7 @@ impl BuilderConfiguration {
 		if ! _outputs.is_dir () {
 			panic! ("[fcee6b4d]");
 		}
-		_outputs
+		normalize_path (&_outputs)
 	}
 }
 
@@ -150,7 +156,7 @@ pub struct Builder {
 	generated : String,
 	counter : u32,
 	route_names : Vec<String>,
-	dependencies : Vec<PathBuf>,
+	dependencies : BTreeSet<PathBuf>,
 }
 
 
@@ -162,7 +168,7 @@ impl Builder {
 				generated : String::with_capacity (1024 * 1024),
 				counter : 0,
 				route_names : Vec::new (),
-				dependencies : Vec::new (),
+				dependencies : BTreeSet::new (),
 			}
 	}
 	
@@ -179,19 +185,45 @@ impl Builder {
 	
 	
 	
+	fn route_asset_raw (&mut self, _relative : String, _source : PathBuf, _content_type : &str, _route_base : Option<&Path>, _route_builder : &(impl RoutePathBuilder + ?Sized), _macro : &str, _from : Option<&Path>) {
+		
+		let _route = _route_builder.build (_relative.as_ref (), &_source, _route_base, None);
+		
+		let _id = self.generate_id ();
+		
+		let _description = if let Some (_from) = _from {
+			format! ("{} ({}, from = `...{}`, file = `...{}`)", _macro, _content_type, _from.display (), _relative)
+		} else {
+			format! ("{} ({}, file = `...{}`)", _macro, _content_type, _relative)
+		};
+		
+		self.route_names.push (format! ("Route_{}", _id));
+		self.dependencies_include (&_source);
+		
+		let _mode = "auto";
+		
+		writeln! (self.generated, "::hyper_static_server::resource! (Resource_{}, {}, {}, (relative_to_cwd, {:?}), {:?});", _id, _content_type, _mode, _source, _description) .unwrap ();
+		writeln! (self.generated, "::hyper_static_server::route! (Route_{}, Resource_{}, {:?});", _id, _id, _route) .unwrap ();
+	}
+	
+	
+	
+	
 	pub fn route_askama (&mut self, _source : &str, _route : &str) -> () {
 		
 		let _askama_sources = self.configuration.askama_sources.as_ref () .expect ("[b4034d6e]");
 		let (_relative, _source) = self.resolve_file (_askama_sources, _source) .expect ("[c1ef5a99]");
 		
+		let _content_type = "Html";
+		
 		let _route = normalize_route (_route.as_ref (), true, false);
 		
 		let _id = self.generate_id ();
-		self.route_names.push (format! ("Route_{}", _id));
-		self.dependencies.push (_source.clone ());
 		
-		let _content_type = "Html";
-		let _description = format! ("askama ({}, source = {:?})", _content_type, _relative);
+		let _description = format! ("askama ({}, source = `...{}`)", _content_type, _relative);
+		
+		self.route_names.push (format! ("Route_{}", _id));
+		self.dependencies_include (&_source);
 		
 		writeln! (self.generated, "::hyper_static_server::askama! (Resource_{}, Template_{}, {}, {:?}, {:?});", _id, _id, _content_type, &_relative[1..], _description) .unwrap ();
 		writeln! (self.generated, "::hyper_static_server::route! (Route_{}, Resource_{}, {:?});", _id, _id, _route) .unwrap ();
@@ -200,181 +232,197 @@ impl Builder {
 	
 	
 	
-	pub fn route_css (&mut self, _source : &str, _route_builder : &impl RoutePathBuilder) -> () {
+	pub fn route_css (&mut self, _source : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
 		
 		let _css_sources = self.configuration.css_sources.as_ref () .expect ("[7844407c]");
 		let (_relative, _source) = self.resolve_file (_css_sources, _source) .expect ("[c6442f7b]");
 		
-		let _id = self.generate_id ();
-		self.route_names.push (format! ("Route_{}", _id));
-		self.dependencies.push (_source.clone ());
+		let _route_base = self.configuration.css_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
 		
-		let _route_base = self.configuration.css_route_base.as_ref () .map (PathBuf::as_path);
-		let _route = _route_builder.build (_relative.as_ref (), &_source, _route_base, None);
-		
-		let _content_type = "Css";
-		let _description = format! ("resource_css ({}, source = {:?})", _content_type, _relative);
-		
-		writeln! (self.generated, "::hyper_static_server::resource! (Resource_{}, {}, embedded, (relative_to_cwd, {:?}), {:?});", _id, _content_type, _source, _description) .unwrap ();
-		writeln! (self.generated, "::hyper_static_server::route! (Route_{}, Resource_{}, {:?});", _id, _id, _route) .unwrap ();
+		self.route_asset_raw (_relative, _source, "Css", _route_base, _route_builder, "resource_css", None);
 	}
 	
 	
-	pub fn route_sass (&mut self, _source : &str, _route_builder : &impl RoutePathBuilder) -> () {
+	pub fn route_sass (&mut self, _source : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
 		
 		let _css_sources = self.configuration.css_sources.as_ref () .expect ("[0d19f056]") .to_owned ();
 		let (_relative, _source) = self.resolve_file (&_css_sources, _source) .expect ("[4f6f6f41]");
 		
-		let _id = self.generate_id ();
-		self.route_names.push (format! ("Route_{}", _id));
-		self.dependencies.push (_source.clone ());
+		self.dependencies_include (&_source);
 		
-		let _relative = PathBuf::from (_relative) .with_extension ("css");
-		
-		let _route_base = self.configuration.css_route_base.as_ref () .map (PathBuf::as_path);
-		let _route = _route_builder.build (_relative.as_ref (), &_source, _route_base, None);
+		let _relative = PathBuf::from (_relative) .with_extension ("css") .to_string_lossy () .into_owned ();
 		
 		let _compiled = self.compile_sass (&_source, &_css_sources) .expect ("[cf9af211]");
 		let _source = self.configuration.outputs.join (fingerprint_data (_source.to_string_lossy ().as_bytes ())) .with_extension ("css");
 		create_file_from_str (&_source, &_compiled) .expect ("[bd7285f4]");
 		
-		let _content_type = "Css";
-		let _description = format! ("resource_sass ({}, source = {:?})", _content_type, _relative);
+		let _route_base = self.configuration.css_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
 		
-		writeln! (self.generated, "::hyper_static_server::resource! (Resource_{}, {}, embedded, (relative_to_cwd, {:?}), {:?});", _id, _content_type, _source, _description) .unwrap ();
-		writeln! (self.generated, "::hyper_static_server::route! (Route_{}, Resource_{}, {:?});", _id, _id, _route) .unwrap ();
+		self.route_asset_raw (_relative, _source.clone (), "Css", _route_base, _route_builder, "resource_sass", None);
+		
+		self.dependencies_exclude (&_source);
 	}
 	
 	
 	
 	
-	pub fn route_js (&mut self, _source : &str, _route_builder : &impl RoutePathBuilder) -> () {
+	pub fn route_js (&mut self, _source : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
 		
 		let _js_sources = self.configuration.js_sources.as_ref () .expect ("[ce97440c]");
 		let (_relative, _source) = self.resolve_file (_js_sources, _source) .expect ("[3acb623e]");
 		
-		let _id = self.generate_id ();
-		self.route_names.push (format! ("Route_{}", _id));
-		self.dependencies.push (_source.clone ());
+		let _route_base = self.configuration.js_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
 		
-		let _route_base = self.configuration.js_route_base.as_ref () .map (PathBuf::as_path);
-		let _route = _route_builder.build (_relative.as_ref (), &_source, _route_base, None);
-		
-		let _content_type = "Js";
-		let _description = format! ("resource_js ({}, path = {:?})", _content_type, _relative);
-		
-		writeln! (self.generated, "::hyper_static_server::resource! (Resource_{}, {}, embedded, (relative_to_cwd, {:?}), {:?});", _id, _content_type, _source, _description) .unwrap ();
-		writeln! (self.generated, "::hyper_static_server::route! (Route_{}, Resource_{}, {:?});", _id, _id, _route) .unwrap ();
+		self.route_asset_raw (_relative, _source, "Js", _route_base, _route_builder, "resource_js", None);
 	}
 	
 	
 	
 	
-	pub fn route_images (&mut self, _sources : &str, _route_builder : &impl RoutePathBuilder) -> () {
+	pub fn route_image (&mut self, _source : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
+		
+		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[24f74a86]");
+		let (_relative, _source) = self.resolve_file (_assets_sources, _source) .expect ("[e606a001]");
+		
+		let _route_base = self.configuration.images_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
+		
+		self.route_image_0 (_relative, _source, _route_base, _route_builder, "resource_image", None);
+	}
+	
+	pub fn route_images (&mut self, _sources : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
 		
 		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[24f74a86]");
 		let (_paths, _folders) = self.resolve_files (_assets_sources, _sources) .expect ("[e606a001]");
-		self.dependencies.extend (_folders);
+		self.dependencies_include_all (_folders.iter () .map (PathBuf::as_path));
 		
 		let _route_base = self.configuration.images_route_base.clone ();
 		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
 		
 		for (_relative, _source) in _paths.into_iter () {
 			
-			let _id = self.generate_id ();
-			
-			self.route_image_0 (_id, _relative, _source, _route_base, _route_builder);
+			self.route_image_0 (_relative, _source, _route_base, _route_builder, "resource_image", Some (_sources.as_ref ()));
 		}
 	}
 	
-	pub fn route_favicons (&mut self, _sources : &str, _route_builder : &impl RoutePathBuilder) -> () {
+	
+	pub fn route_icon (&mut self, _source : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
+		
+		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[24f74a86]");
+		let (_relative, _source) = self.resolve_file (_assets_sources, _source) .expect ("[e606a001]");
+		
+		let _route_base = self.configuration.icons_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
+		
+		self.route_image_0 (_relative, _source, _route_base, _route_builder, "resource_icon", None);
+	}
+	
+	pub fn route_icons (&mut self, _sources : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
 		
 		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[24f74a86]");
 		let (_paths, _folders) = self.resolve_files (_assets_sources, _sources) .expect ("[e606a001]");
-		self.dependencies.extend (_folders);
+		self.dependencies_include_all (_folders.iter () .map (PathBuf::as_path));
+		
+		let _route_base = self.configuration.icons_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
+		
+		for (_relative, _source) in _paths.into_iter () {
+			
+			self.route_image_0 (_relative, _source, _route_base, _route_builder, "resource_icon", Some (_sources.as_ref ()));
+		}
+	}
+	
+	
+	pub fn route_favicon (&mut self, _source : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
+		
+		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[24f74a86]");
+		let (_relative, _source) = self.resolve_file (_assets_sources, _source) .expect ("[e606a001]");
+		
+		let _route_base = self.configuration.favicons_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
+		
+		self.route_image_0 (_relative, _source, _route_base, _route_builder, "resource_favicon", None);
+	}
+	
+	pub fn route_favicons (&mut self, _sources : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
+		
+		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[24f74a86]");
+		let (_paths, _folders) = self.resolve_files (_assets_sources, _sources) .expect ("[e606a001]");
+		self.dependencies_include_all (_folders.iter () .map (PathBuf::as_path));
 		
 		let _route_base = self.configuration.favicons_route_base.clone ();
 		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
 		
 		for (_relative, _source) in _paths.into_iter () {
 			
-			let _id = self.generate_id ();
-			
-			self.route_image_0 (_id, _relative, _source, _route_base, _route_builder);
+			self.route_image_0 (_relative, _source, _route_base, _route_builder, "resource_favicon", Some (_sources.as_ref ()));
 		}
 	}
 	
-	fn route_image_0 (&mut self, _id : u32, _relative : String, _source : PathBuf, _route_base : Option<&Path>, _route_builder : &impl RoutePathBuilder) -> () {
+	
+	fn route_image_0 (&mut self, _relative : String, _source : PathBuf, _route_base : Option<&Path>, _route_builder : &(impl RoutePathBuilder + ?Sized), _macro : &str, _from : Option<&Path>) -> () {
 		
-		let _route = _route_builder.build (_relative.as_ref (), &_source, _route_base, None);
-		
-		self.route_names.push (format! ("Route_{}", _id));
-		self.dependencies.push (_source.clone ());
-		
-		let _extension = _source.extension () .expect ("[4b7affbe]") .to_str () .expect ("[a874a519]");
-		let _content_type = match _extension {
-			"png" => "Png",
-			"jpeg" | "jpg" => "Jpeg",
-			"ico" => "Icon",
-			"svg" => "Svg",
+		let _content_type = detect_content_type_from_extension (&_source);
+		match _content_type {
+			"Png" | "Jpeg" | "Icon" | "Svg" =>
+				(),
 			_ =>
-				panic! ("[0fd2d804] {:?}", _source),
+				panic! ("[0fd2d804] {}", _source.display ()),
 		};
 		
-		let _description = format! ("resource_image ({}, path = {:?})", _content_type, _relative);
-		
-		writeln! (self.generated, "::hyper_static_server::resource! (Resource_{}, {}, embedded, (relative_to_cwd, {:?}), {:?});", _id, _content_type, _source, _description) .unwrap ();
-		writeln! (self.generated, "::hyper_static_server::route! (Route_{}, Resource_{}, {:?});", _id, _id, _route) .unwrap ();
+		self.route_asset_raw (_relative, _source, _content_type, _route_base, _route_builder, _macro, _from);
 	}
 	
 	
 	
 	
-	pub fn route_fonts (&mut self, _sources : &str, _route_builder : &impl RoutePathBuilder) -> () {
+	pub fn route_font (&mut self, _source : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
+		
+		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[24f74a86]");
+		let (_relative, _source) = self.resolve_file (_assets_sources, _source) .expect ("[e606a001]");
+		
+		let _route_base = self.configuration.fonts_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
+		
+		self.route_font_0 (_relative, _source, _route_base, _route_builder, None);
+	}
+	
+	pub fn route_fonts (&mut self, _sources : &str, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
 		
 		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[24f74a86]");
 		let (_paths, _folders) = self.resolve_files (_assets_sources, _sources) .expect ("[5ffa5360]");
-		self.dependencies.extend (_folders);
+		self.dependencies_include_all (_folders.iter () .map (PathBuf::as_path));
 		
 		let _route_base = self.configuration.fonts_route_base.clone ();
 		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
 		
 		for (_relative, _source) in _paths.into_iter () {
 			
-			let _id = self.generate_id ();
-			
-			self.route_font_0 (_id, _relative, _source, _route_base, _route_builder);
+			self.route_font_0 (_relative, _source, _route_base, _route_builder, Some (_sources.as_ref ()));
 		}
 	}
 	
 	
-	fn route_font_0 (&mut self, _id : u32, _relative : String, _source : PathBuf, _route_base : Option<&Path>, _route_builder : &impl RoutePathBuilder) -> () {
+	fn route_font_0 (&mut self, _relative : String, _source : PathBuf, _route_base : Option<&Path>, _route_builder : &(impl RoutePathBuilder + ?Sized), _from : Option<&Path>) -> () {
 		
-		self.route_names.push (format! ("Route_{}", _id));
-		self.dependencies.push (_source.clone ());
-		
-		let _route = _route_builder.build (_relative.as_ref (), &_source, _route_base, None);
-		
-		let _extension = _source.extension () .expect ("[d4f695e7]") .to_str () .expect ("[53aff957]");
-		let _content_type = match _extension {
-			"ttf" => "FontTtf",
-			"otf" => "FontOtf",
-			"woff" => "FontWoff",
-			"woff2" => "FontWoff2",
+		let _content_type = detect_content_type_from_extension (&_source);
+		match _content_type {
+			"FontTtf" | "FontOtf" | "FontWoff" | "FontWoff2" =>
+				(),
 			_ =>
-				panic! ("[1a4ccbf4] {:?}", _source),
+				panic! ("[1a4ccbf4] {}", _source.display ()),
 		};
 		
-		let _description = format! ("resource_font ({}, source = {:?})", _content_type, _relative);
-		
-		writeln! (self.generated, "::hyper_static_server::resource! (Resource_{}, {}, embedded, (relative_to_cwd, {:?}), {:?});", _id, _content_type, _source, _description) .unwrap ();
-		writeln! (self.generated, "::hyper_static_server::route! (Route_{}, Resource_{}, {:?});", _id, _id, _route) .unwrap ();
+		self.route_asset_raw (_relative, _source, _content_type, _route_base, _route_builder, "resource_font", _from);
 	}
 	
 	
 	
 	
-	pub fn route_asset (&mut self, _source : &str, _content_type : Option<&str>, _route : Option<&str>, _route_builder : &impl RoutePathBuilder) {
+	pub fn route_asset (&mut self, _source : &str, _content_type : Option<&str>, _route_builder : &(impl RoutePathBuilder + ?Sized)) {
 		
 		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[443c8ae5]");
 		let (_relative, _source) = self.resolve_file (_assets_sources, _source) .expect ("[256c60bf]");
@@ -382,71 +430,30 @@ impl Builder {
 		let _route_base = self.configuration.assets_route_base.clone ();
 		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
 		
-		let _id = self.generate_id ();
-		
-		self.route_asset_0 (_id, _relative, _source, _content_type, _route.as_ref () .map (Path::new), _route_base, None, _route_builder);
+		self.route_asset_0 (_relative, _source, _content_type, _route_base, _route_builder, None);
 	}
 	
-	
-	pub fn route_assets (&mut self, _sources : &str, _content_type : Option<&str>, _route_base : Option<&str>, _route_builder : &impl RoutePathBuilder) -> () {
+	pub fn route_assets (&mut self, _sources : &str, _content_type : Option<&str>, _route_builder : &(impl RoutePathBuilder + ?Sized)) -> () {
 		
 		let _assets_sources = self.configuration.assets_sources.as_ref () .expect ("[d807eb26]");
 		let (_paths, _folders) = self.resolve_files (_assets_sources, _sources) .expect ("[5ffa5360]");
-		self.dependencies.extend (_folders);
+		self.dependencies_include_all (_folders.iter () .map (PathBuf::as_path));
 		
-		let _route_base_0 = self.configuration.assets_route_base.clone ();
-		let _route_base_0 = _route_base_0.as_ref () .map (PathBuf::as_path);
-		let _route_base = _route_base.map (Path::new);
-		let _route_base = _route_base.or (_route_base_0);
+		let _route_base = self.configuration.assets_route_base.clone ();
+		let _route_base = _route_base.as_ref () .map (PathBuf::as_path);
 		
 		for (_relative, _source) in _paths.into_iter () {
 			
-			let _id = self.generate_id ();
-			
-			self.route_asset_0 (_id, _relative, _source, _content_type, None, _route_base, None, _route_builder);
+			self.route_asset_0 (_relative, _source, _content_type, _route_base, _route_builder, Some (_sources.as_ref ()));
 		}
 	}
 	
 	
-	fn route_asset_0 (&mut self, _id : u32, _relative : String, _source : PathBuf, _content_type : Option<&str>, _route : Option<&Path>, _route_base : Option<&Path>, _route_infix : Option<&Path>, _route_builder : &impl RoutePathBuilder) {
+	fn route_asset_0 (&mut self, _relative : String, _source : PathBuf, _content_type : Option<&str>, _route_base : Option<&Path>, _route_builder : &(impl RoutePathBuilder + ?Sized), _from : Option<&Path>) {
 		
-		self.route_names.push (format! ("Route_{}", _id));
-		self.dependencies.push (_source.clone ());
+		let _content_type = _content_type.unwrap_or_else (|| detect_content_type_from_extension (&_source));
 		
-		let _route = if let Some (_route) = _route {
-			normalize_route (_route, false, false)
-		} else {
-			_route_builder.build (_relative.as_ref (), &_source, _route_base, _route_infix)
-		};
-		
-		let _extension = _source.extension () .expect ("[29957dc8]") .to_str () .expect ("[908aeea6]");
-		let _content_type = if let Some (_content_type) = _content_type {
-			_content_type
-		} else {
-			match _extension {
-				"text" | "txt" => "Text",
-				"html" | "htm" => "Html",
-				"css" => "Css",
-				"js" => "Js",
-				"json" => "Json",
-				"xml" => "Xml",
-				"png" => "Png",
-				"jpeg" | "jpg" => "Jpeg",
-				"ico" => "Icon",
-				"svg" => "Svg",
-				"ttf" => "FontTtf",
-				"otf" => "FontOtf",
-				"woff" => "FontWoff",
-				"woff2" => "FontWoff2",
-				_ =>
-					panic! ("[2bd15bab] {:?}", _source),
-			}
-		};
-		
-		let _description = format! ("resource_asset ({}, path = {:?})", _content_type, _relative);
-		
-		writeln! (self.generated, "::hyper_static_server::resource! (Resource_{}, {}, embedded, (relative_to_cwd, {:?}), {:?});", _id, _content_type, _source, _description) .unwrap ();
-		writeln! (self.generated, "::hyper_static_server::route! (Route_{}, Resource_{}, {:?});", _id, _id, _route) .unwrap ();
+		self.route_asset_raw (_relative, _source, _content_type, _route_base, _route_builder, "resource_asset", _from);
 	}
 	
 	
@@ -454,12 +461,20 @@ impl Builder {
 	
 	pub fn generate (mut self) -> () {
 		
+		self.dependencies_extend ();
+		
 		writeln! (self.generated, "::hyper_static_server::routes! (Routes, [") .unwrap ();
 		for _route_name in self.route_names.into_iter () {
 			writeln! (self.generated, "\t{},", _route_name) .unwrap ();
 		}
 		writeln! (self.generated, "]);") .unwrap ();
 		
+		writeln! (self.generated, "::hyper_static_server::dependencies! (Dependencies, [") .unwrap ();
+		for _dependency in self.dependencies.iter () {
+			writeln! (self.generated, "\t{:?},", _dependency) .unwrap ();
+		}
+		writeln! (self.generated, "]);") .unwrap ();
+		
 		create_file_from_str (&self.configuration.generated, &self.generated) .expect ("[9796aa67]");
 		
 		if false {
@@ -493,12 +508,14 @@ impl Builder {
 		let _path = _root.join (&_source[1..]);
 		
 		if ! _path.is_file () {
-			return Err (io::Error::new (io::ErrorKind::Other, format! ("[039d945b] {:?}", _path)));
+			return Err (io::Error::new (io::ErrorKind::Other, format! ("[039d945b] {}", _path.display ())));
 		}
 		
 		let _relative = _path.strip_prefix (_root) .expect ("[546e7cd9]") .to_str () .expect ("[a48f283c]");
 		let _relative = ["/", _relative].concat ();
 		
+		let _path = normalize_path (&_path);
+		
 		return Ok ((_relative.into (), _path));
 	}
 	
@@ -525,12 +542,21 @@ impl Builder {
 		for _entry in walkdir::WalkDir::new (&_root) {
 			let _entry = _entry ?;
 			let _path = _entry.path ();
+			
 			if _path.is_file () {
+				
 				let _relative = _path.strip_prefix (&_root) .expect ("[703112d8]") .to_str () .expect ("[072ffc03]");
 				let _relative = ["/", _relative].concat ();
-				_paths.push ((_relative.into (), _path.into ()));
+				
+				let _path = normalize_path (_path);
+				
+				_paths.push ((_relative.into (), _path));
 			}
+			
 			if _path.is_dir () {
+				
+				let _path = normalize_path (_path);
+				
 				_folders.push (_path.into ());
 			}
 		}
@@ -539,6 +565,44 @@ impl Builder {
 	}
 	
 	
+	
+	
+	fn dependencies_include (&mut self, _path : &Path) -> () {
+		self.dependencies.insert (_path.into ());
+	}
+	
+	fn dependencies_include_all <'a> (&mut self, _paths : impl Iterator<Item = &'a Path>) -> () {
+		for _path in _paths {
+			self.dependencies_include (_path);
+		}
+	}
+	
+	fn dependencies_exclude (&mut self, _path : &Path) -> () {
+		self.dependencies.remove (_path.into ());
+	}
+	
+	fn dependencies_extend (&mut self) -> () {
+		
+		let mut _extra = Vec::new ();
+		
+		for _dependency in self.dependencies.iter () {
+			
+			let _metadata = fs::symlink_metadata (_dependency) .expect ("[e5c6e436]");
+			
+			if _metadata.file_type () .is_symlink () {
+				let _target = _dependency.canonicalize () .expect ("[12da2ba1]");
+				_extra.push (_target);
+			}
+		}
+		
+		for _dependency in _extra.into_iter () {
+			self.dependencies.insert (_dependency);
+		}
+	}
+	
+	
+	
+	
 	fn generate_id (&mut self) -> u32 {
 		let _id = self.counter;
 		self.counter += 1;
@@ -587,42 +651,59 @@ fn create_file_from_str (_path : &Path, _data : &str) -> Result<(), io::Error> {
 
 pub trait RoutePathBuilder {
 	
-	fn build (&self, _source_relative : &Path, _source_path : &Path, _route_base_hint : Option<&Path>, _route_infix_hint : Option<&Path>) -> PathBuf;
+	fn build (&self, _source_relative : &Path, _source_path : &Path, _route_prefix_hint : Option<&Path>, _route_infix_hint : Option<&Path>) -> PathBuf;
 }
 
 
 impl RoutePathBuilder for () {
 	
-	fn build (&self, _source_relative : &Path, _source_path : &Path, _route_base_hint : Option<&Path>, _route_infix : Option<&Path>) -> PathBuf {
-		
-		let _route_base = _route_base_hint.expect ("[1ba00780]");
-		
-		if ! _route_base.starts_with ("/") || _route_base.ends_with ("/") {
-			panic! ("[6fc9256c]");
-		}
-		if ! _source_relative.starts_with ("/") || _source_relative.ends_with ("/") {
-			panic! ("[ace09af4]");
-		}
-		if let Some (_route_infix) = _route_infix {
-			if _route_infix.starts_with ("/") || _route_infix.ends_with ("/") {
-				panic! ("[d224b592]");
-			}
-		}
-		
-		let _source_relative = _source_relative.strip_prefix ("/") .expect ("[bd4b80bd]");
-		
-		let _route = if let Some (_route_infix) = _route_infix {
-			let _route_infix = _route_infix.strip_prefix ("/") .expect ("[1a7e3353]");
-			_route_base.join (_route_infix) .join (_source_relative)
-		} else {
-			_route_base.join (_source_relative)
-		};
-		
-		normalize_route (&_route, false, false)
+	fn build (&self, _source_relative : &Path, _source_path : &Path, _route_prefix_hint : Option<&Path>, _route_infix_hint : Option<&Path>) -> PathBuf {
+		generate_route (_source_relative, _route_prefix_hint, _route_infix_hint)
 	}
 }
 
 
+impl RoutePathBuilder for (bool, &str) {
+	
+	fn build (&self, _source_relative : &Path, _source_path : &Path, _route_prefix_hint : Option<&Path>, _route_infix_hint : Option<&Path>) -> PathBuf {
+		if self.0 {
+			generate_route (_source_relative, Some (Path::new (self.1)), None)
+		} else {
+			normalize_route (Path::new (self.1), true, false)
+		}
+	}
+}
+
+
+
+
+fn generate_route (_source_relative : &Path, _route_prefix : Option<&Path>, _route_infix : Option<&Path>) -> PathBuf {
+	
+	let _route_prefix = _route_prefix.expect ("[1ba00780]");
+	
+	if ! _route_prefix.starts_with ("/") || _route_prefix.ends_with ("/") {
+		panic! ("[6fc9256c]");
+	}
+	if ! _source_relative.starts_with ("/") || _source_relative.ends_with ("/") {
+		panic! ("[ace09af4]");
+	}
+	if let Some (_route_infix) = _route_infix {
+		if _route_infix.starts_with ("/") || _route_infix.ends_with ("/") {
+			panic! ("[d224b592]");
+		}
+	}
+	
+	let _source_relative = _source_relative.strip_prefix ("/") .expect ("[bd4b80bd]");
+	
+	let _route = if let Some (_route_infix) = _route_infix {
+		let _route_infix = _route_infix.strip_prefix ("/") .expect ("[1a7e3353]");
+		_route_prefix.join (_route_infix) .join (_source_relative)
+	} else {
+		_route_prefix.join (_source_relative)
+	};
+	
+	normalize_route (&_route, false, false)
+}
 
 
 fn normalize_route (_path_0 : &Path, _keep_trailing_slash : bool, _force_trailing_slash : bool) -> PathBuf {
@@ -649,6 +730,43 @@ fn normalize_route (_path_0 : &Path, _keep_trailing_slash : bool, _force_trailin
 }
 
 
+fn normalize_path (_path_0 : &Path) -> PathBuf {
+	
+	let mut _path = PathBuf::new ();
+	for _component in _path_0.components () {
+		_path.push (_component);
+	}
+	_path
+}
+
+
+
+
+fn detect_content_type_from_extension (_source : &Path) -> &'static str {
+	
+	let _extension = _source.extension () .expect ("[29957dc8]") .to_str () .expect ("[908aeea6]");
+	
+	match _extension {
+		"text" | "txt" => "Text",
+		"html" | "htm" => "Html",
+		"css" => "Css",
+		"js" => "Js",
+		"json" => "Json",
+		"xml" => "Xml",
+		"png" => "Png",
+		"jpeg" | "jpg" => "Jpeg",
+		"ico" => "Icon",
+		"svg" => "Svg",
+		"ttf" => "FontTtf",
+		"otf" => "FontOtf",
+		"woff" => "FontWoff",
+		"woff2" => "FontWoff2",
+		_ =>
+			panic! ("[2bd15bab] {}", _source.display ()),
+	}
+}
+
+
 
 
 fn fingerprint_data (_data : impl AsRef<[u8]>) -> String {
@@ -659,97 +777,3 @@ fn fingerprint_data (_data : impl AsRef<[u8]>) -> String {
 	format! ("{:x}", _hash)
 }
 
-
-
-
-
-
-
-
-#[ macro_export ]
-macro_rules! builder_macros {
-	
-	
-	( $_builder : ident ) => {
-		
-		
-		#[ allow (unused_macros) ]
-		macro_rules! askama {
-			( $_source : literal => $_route : literal ) => {
-				$_builder.route_askama ($_source, $_route);
-			};
-		}
-		
-		
-		#[ allow (unused_macros) ]
-		macro_rules! asset_css {
-			( $_source : literal ) => {
-				$_builder.route_css ($_source, &());
-			};
-		}
-		
-		#[ allow (unused_macros) ]
-		macro_rules! asset_sass {
-			( $_source : literal ) => {
-				$_builder.route_sass ($_source, &());
-			};
-		}
-		
-		#[ allow (unused_macros) ]
-		macro_rules! asset_js {
-			( $_source : literal ) => {
-				$_builder.route_js ($_source, &());
-			};
-		}
-		
-		
-		#[ allow (unused_macros) ]
-		macro_rules! assets_images {
-			( $_sources : literal ) => {
-				$_builder.route_images ($_sources, &());
-			};
-		}
-		
-		#[ allow (unused_macros) ]
-		macro_rules! assets_favicons {
-			( $_sources : literal ) => {
-				$_builder.route_favicons ($_sources, &());
-			};
-		}
-		
-		#[ allow (unused_macros) ]
-		macro_rules! assets_fonts {
-			( $_sources : literal ) => {
-				$_builder.route_fonts ($_sources, &());
-			};
-		}
-		
-		
-		#[ allow (unused_macros) ]
-		macro_rules! asset {
-			( $_source : literal => $_route : literal ) => {
-				asset! (internal, $_source, $_route, ::std::option::Option::None)
-			};
-			( $_source : literal : $_content_type : ident => $_route : literal ) => {
-				asset! (internal, $_source, $_route, ::std::option::Option::Some (std::stringify! ($_content_type)))
-			};
-			( internal, $_source : literal, $_route : literal, $_content_type : expr ) => {
-				$_builder.route_asset ($_source, $_content_type, ::std::option::Option::Some ($_route), &());
-			}
-		}
-		
-		#[ allow (unused_macros) ]
-		macro_rules! assets {
-			( $_source : literal => $_route : literal ) => {
-				assets! (internal, $_source, $_route, ::std::option::Option::None)
-			};
-			( $_source : literal : $_content_type : ident => $_route : literal ) => {
-				assets! (internal, $_source, $_route, ::std::option::Option::Some (std::stringify! ($_content_type)))
-			};
-			( internal, $_source : literal, $_route : literal, $_content_type : expr ) => {
-				$_builder.route_assets ($_source, $_content_type, ::std::option::Option::Some ($_route), &());
-			}
-		}
-	};
-}
-
